@@ -3,10 +3,12 @@
 
 import os
 import subprocess, sys
+import io
 from io import StringIO
 import inspect
 from pathlib import PurePath
 from threading import Thread
+import pdb
 
 from .exceptions import InvalidTask, TaskFailed, TaskError
 
@@ -47,10 +49,10 @@ class BaseAction(object):
 
         # use task meta information as extra_args
         meta_args = {
-            'task': task,
-            'targets': task.targets,
-            'dependencies': task.file_dep,
-            'changed': task.dep_changed,
+            'task': lambda: task,
+            'targets': lambda: list(task.targets),
+            'dependencies': lambda: list(task.file_dep),
+            'changed': lambda: list(task.dep_changed),
             }
 
         # start with dict passed together on action definition
@@ -73,7 +75,7 @@ class BaseAction(object):
 
                 # if value not taken from position parameter
                 if key not in bound_args.arguments:
-                    kwargs[key] = meta_args[key]
+                    kwargs[key] = meta_args[key]()
 
 
         # add tasks parameter options
@@ -114,6 +116,8 @@ class CmdAction(BaseAction):
     @ivar pkwargs: Popen arguments except 'stdout' and 'stderr'
     """
 
+    STRING_FORMAT = 'old'
+
     def __init__(self, action, task=None, save_out=None, shell=True,
                  encoding='utf-8', decode_error='replace', buffering=0,
                  **pkwargs): #pylint: disable=W0231
@@ -209,29 +213,37 @@ class CmdAction(BaseAction):
                 env = os.environ.copy()
             env['PYTHONUNBUFFERED'] = '1'
 
+        capture_io = self.task.io.capture if self.task else True
+        if capture_io:
+            p_out = p_err = subprocess.PIPE
+        else:
+            p_out = p_err = None
+
         # spawn task process
         process = subprocess.Popen(
             action,
             shell=self.shell,
             #bufsize=2, # ??? no effect use PYTHONUNBUFFERED instead
-            stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+            stdout=p_out,
+            stderr=p_err,
             env=env,
             **subprocess_pkwargs)
 
-        output = StringIO()
-        errput = StringIO()
-        t_out = Thread(target=self._print_process_output,
-                       args=(process, process.stdout, output, out))
-        t_err = Thread(target=self._print_process_output,
-                       args=(process, process.stderr, errput, err))
-        t_out.start()
-        t_err.start()
-        t_out.join()
-        t_err.join()
+        if capture_io:
+            output = StringIO()
+            errput = StringIO()
+            t_out = Thread(target=self._print_process_output,
+                           args=(process, process.stdout, output, out))
+            t_err = Thread(target=self._print_process_output,
+                           args=(process, process.stderr, errput, err))
+            t_out.start()
+            t_err.start()
+            t_out.join()
+            t_err.join()
 
-        self.out = output.getvalue()
-        self.err = errput.getvalue()
-        self.result = self.out + self.err
+            self.out = output.getvalue()
+            self.err = errput.getvalue()
+            self.result = self.out + self.err
 
         # make sure process really terminated
         process.wait()
@@ -297,7 +309,14 @@ class CmdAction(BaseAction):
             else:
                 pos_val = ''
             subs_dict[self.task.pos_arg] = pos_val
-        return self.action % subs_dict
+
+        if self.STRING_FORMAT == 'old':
+            return self.action % subs_dict
+        elif self.STRING_FORMAT == 'new':
+            return self.action.format(**subs_dict)
+        else:
+            assert self.STRING_FORMAT == 'both'
+            return self.action.format(**subs_dict) % subs_dict
 
     def __str__(self):
         return "Cmd: %s" % self._action
@@ -309,21 +328,24 @@ class CmdAction(BaseAction):
 
 
 class Writer(object):
-    """write to many streams"""
+    """Write to N streams.
+
+    This is used on python-actions to allow the stream to be output to terminal and captured at the same time.
+    """
     def __init__(self, *writers):
         """@param writers - file stream like objects"""
         self.writers = []
-        self._isatty = True
+        self.orig_stream = None # The original stream terminal/file
         for writer in writers:
             self.add_writer(writer)
 
-    def add_writer(self, stream, isatty=None):
+    def add_writer(self, stream, *, is_original=False):
         """adds a stream to the list of writers
-        @param isatty: (bool) if specified overwrites real isatty from stream
+        @param is: (bool) if specified overwrites real isatty from stream
         """
         self.writers.append(stream)
-        isatty = stream.isatty() if (isatty is None) else isatty
-        self._isatty = self._isatty and isatty
+        if is_original:
+            self.orig_stream = stream
 
     def write(self, text):
         """write 'text' to all streams"""
@@ -336,7 +358,14 @@ class Writer(object):
             stream.flush()
 
     def isatty(self):
-        return self._isatty
+        if self.orig_stream:
+            return self.orig_stream.isatty()
+        return False
+
+    def fileno(self):
+        if self.orig_stream:
+            return self.orig_stream.fileno()
+        raise io.UnsupportedOperation()
 
 
 class PythonAction(BaseAction):
@@ -346,7 +375,10 @@ class PythonAction(BaseAction):
     @ivar args: (sequence)  Extra arguments to be passed to py_callable
     @ivar kwargs: (dict) Extra keyword arguments to be passed to py_callable
     @ivar task(Task): reference to task that contains this action
+    @ivar pm_pdb: if True drop into PDB on exception when executing task
     """
+    pm_pdb = False
+
     def __init__(self, py_callable, args=None, kwargs=None, task=None):
         #pylint: disable=W0231
         self.py_callable = py_callable
@@ -399,23 +431,26 @@ class PythonAction(BaseAction):
 
         @return failure: see CmdAction.execute
         """
-        # set std stream
-        old_stdout = sys.stdout
-        output = StringIO()
-        out_writer = Writer()
-        # capture output but preserve isatty() from original stream
-        out_writer.add_writer(output, old_stdout.isatty())
-        if out:
-            out_writer.add_writer(out)
-        sys.stdout = out_writer
+        capture_io = self.task.io.capture if self.task else True
 
-        old_stderr = sys.stderr
-        errput = StringIO()
-        err_writer = Writer()
-        err_writer.add_writer(errput, old_stderr.isatty())
-        if err:
-            err_writer.add_writer(err)
-        sys.stderr = err_writer
+        if capture_io:
+            # set std stream
+            old_stdout = sys.stdout
+            output = StringIO()
+            out_writer = Writer()
+            # capture output but preserve isatty() from original stream
+            out_writer.add_writer(output)
+            if out:
+                out_writer.add_writer(out, is_original=True)
+            sys.stdout = out_writer
+
+            old_stderr = sys.stderr
+            errput = StringIO()
+            err_writer = Writer()
+            err_writer.add_writer(errput)
+            if err:
+                err_writer.add_writer(err, is_original=True)
+            sys.stderr = err_writer
 
         kwargs = self._prepare_kwargs()
 
@@ -423,13 +458,19 @@ class PythonAction(BaseAction):
         try:
             returned_value = self.py_callable(*self.args, **kwargs)
         except Exception as exception:
+            if self.pm_pdb: # pragma: no cover
+                # start post-mortem debugger
+                deb = pdb.Pdb(stdin=sys.__stdin__, stdout=sys.__stdout__)
+                deb.reset()
+                deb.interaction(None, sys.exc_info()[2])
             return TaskError("PythonAction Error", exception)
         finally:
             # restore std streams /log captured streams
-            sys.stdout = old_stdout
-            sys.stderr = old_stderr
-            self.out = output.getvalue()
-            self.err = errput.getvalue()
+            if capture_io:
+                sys.stdout = old_stdout
+                sys.stderr = old_stderr
+                self.out = output.getvalue()
+                self.err = errput.getvalue()
 
         # if callable returns false. Task failed
         if returned_value is False:
